#!/usr/bin/env python3
import tqdm
import tqdm.notebook
from typing import Union
import numpy as np
import pandas as pd
import matplotlib
import string
from pathlib import Path


def colorbrewer(name: str):
    """Get all colors from a colormap in matplotlib

    Args:
        name: Name of the colormap
    """
    cmap = matplotlib.colormaps[name]
    colors = pd.DataFrame(
        cmap(np.linspace(0, 1, cmap.N)),
        index=range(1, cmap.N + 1),
        columns=["R", "G", "B", "A"],
    )
    return colors


def _in_notebook() -> bool:
    """Check if code is run in IPython notebook.
    The variable `__IPYTHON__` is defined in Jupyter or IPython but not a normal Python interpreter.

    Returns:
        `True` if code is executed from a Jupyter/IPython notebook
    """
    try:
        __IPYTHON__  # type: ignore
        return True
    except NameError:
        return False


def _initialize_pbar(
    n: int, msg: str
) -> Union[tqdm.std.tqdm, tqdm.notebook.tqdm_notebook]:
    """Initialize a progress bar

    Args:
        n: Number of steps
        msg: Progress bar message

    Returns:
        A tqdm progress bar
    """
    if _in_notebook():
        pbar = tqdm.notebook.tqdm(
            total=n,
            desc=msg,
            bar_format="{desc}: {n_fmt}/{total_fmt} [{remaining} s]",
        )
    else:
        pbar = tqdm.tqdm(
            total=n,
            desc=msg,
            bar_format="{desc}: {n_fmt}/{total_fmt} [{remaining} s]",
        )
    return pbar


def alpha_to_numeric(alpha: str) -> int:
    """Return the position of a single character in the alphabet

    Args:
        alpha: Single alphabet character

    Returns:
        Integer position in the alphabet
    """
    return ord(alpha.upper()) - 64


def numeric_to_alpha(numeric: int, upper: bool = True) -> str:
    """Return the upper or lowercase character for a given position in the alphabet

    Args:
        numeric: Integer position in the alphabet

    Returns:
        Single alphabet character
    """
    if upper:
        string.ascii_uppercase[numeric - 1]
    else:
        string.ascii_lowercase[numeric - 1]



import numpy as np
import pandas as pd
import logging
from pathlib import Path

import fractal_tasks_core
from pydantic import validate_call

from operetta_compose import io

__OME_NGFF_VERSION__ = fractal_tasks_core.__OME_NGFF_VERSION__

logger = logging.getLogger(__name__)


@validate_call
def condition_registration(
    *,
    zarr_url: str,
    layout_path: str,
    condition_name: str = "condition",
    overwrite: bool = False,
) -> None:
    """Register the experimental (drug layout) in the OME-ZARR

    Args:
        zarr_url: Path to an OME-ZARR Image
        layout_path: Path to a drug layout file (.csv) with at least the columns: row, col, drug, concentration and unit
        condition_name: Name of the condition table
        overwrite: Whether to overwrite any existing OME-ZARR condition table
    """
    condition_dir = Path(f"{zarr_url}/tables/{condition_name}")
    if (not condition_dir.is_dir()) | overwrite:
        layout = pd.read_csv(layout_path, sep=None, engine="python")
        layout["col"] = layout["col"].astype(str)
        ome_zarr_url = io.parse_zarr_url(zarr_url)
        condition_table = layout.query(
            "row == @ome_zarr_url.row & col == @ome_zarr_url.col"
        )
        if not condition_table.empty:
            io.condition_to_ome_zarr(zarr_url, condition_table, condition_name)
            io.write_table_metadata(zarr_url, "condition_table", "condition")
        else:
            logger.warning(
                "Well {row}{col} of the OME-ZARR fileset is not found in the drug layout"
            )
    else:
        raise FileExistsError(
            f"{zarr_url} already contains a condition table in the OME-ZARR fileset. To ignore the existing table set `overwrite = True`."
        )


if __name__ == "__main__":
    from fractal_tasks_core.tasks._utils import run_fractal_task

    run_fractal_task(
        task_function=condition_registration,
        logger_name=logger.name,
    )



import pytest
from pathlib import Path

from fractal_tasks_core.channels import ChannelInputModel

from operetta_compose.tasks.harmony_to_ome_zarr import harmony_to_ome_zarr
from operetta_compose.tasks.stardist_segmentation import stardist_segmentation
from operetta_compose.tasks.regionprops_measurement import regionprops_measurement
from operetta_compose.tasks.label_prediction import label_prediction
from operetta_compose.tasks.condition_registration import condition_registration

TEST_DIR = Path(__file__).resolve().parent
ZARR_DIR = Path(TEST_DIR).joinpath("test_output")
PLATE = "operetta_plate"
PLATE_ZARR = PLATE + ".zarr"


@pytest.fixture
def _make_output_dir():
    zarr_dir = Path(ZARR_DIR)
    zarr_dir.mkdir(parents=True, exist_ok=True)


@pytest.mark.dependency()
def test_converter(_make_output_dir):
    harmony_to_ome_zarr(
        zarr_urls=[],
        img_paths=[str(Path(TEST_DIR).joinpath(PLATE, "Images"))],
        zarr_dir=str(ZARR_DIR),
        overwrite=True,
        compute=True,
    )


@pytest.mark.dependency(depends=["test_converter"])
def test_stardist():
    stardist_segmentation(
        zarr_url=str(ZARR_DIR.joinpath(PLATE_ZARR, "C", "3", "0")),
        channel=ChannelInputModel(label="Fluorescein (FITC)"),
        roi_table="FOV_ROI_table",
        stardist_model="2D_versatile_fluo",
        label_name="nuclei",
        prob_thresh=None,
        nms_thresh=None,
        scale=1,
        level=0,
        overwrite=True,
    )


@pytest.mark.dependency(depends=["test_converter", "test_stardist"])
def test_measure():
    regionprops_measurement(
        zarr_url=str(ZARR_DIR.joinpath(PLATE_ZARR, "C", "3", "0")),
        table_name="regionprops",
        label_name="nuclei",
        level=0,
        overwrite=True,
    )


@pytest.mark.dependency(depends=["test_converter", "test_stardist", "test_measure"])
# @pytest.mark.skip
def test_predict():
    label_prediction(
        zarr_url=str(ZARR_DIR.joinpath(PLATE_ZARR, "C", "3", "0")),
        classifier_path=str(Path(TEST_DIR).joinpath("classifier.pkl")),
        table_name="regionprops",
        label_name="nuclei",
    )


@pytest.mark.dependency(depends=["test_converter"])
def test_register_layout():
    condition_registration(
        zarr_url=str(ZARR_DIR.joinpath(PLATE_ZARR, "C", "3", "0")),
        layout_path=str(Path(TEST_DIR).joinpath("drug_layout.csv")),
        condition_name="condition",
        overwrite=True,
    )


if __name__ == "__main__":
    pytest.main()



import numpy as np
import pandas as pd
import anndata as ad
import logging
import pickle
from zarr.errors import PathNotFoundError

import fractal_tasks_core
from pydantic import validate_call

from operetta_compose import io


__OME_NGFF_VERSION__ = fractal_tasks_core.__OME_NGFF_VERSION__

logger = logging.getLogger(__name__)


@validate_call
def label_prediction(
    *,
    zarr_url: str,
    classifier_path: str,
    table_name: str = "regionprops",
    label_name: str = "nuclei",
) -> None:
    """Make predictions on the selected wells and write them to the OME-ZARR

    Args:
        zarr_url: Path to an OME-ZARR Image
        classifier_path: Path to the pickled scikit-learn classifier
        table_name: Folder name of the measured regionprobs features
        label_name: Name of the labels to use for feature measurements
    """
    with open(classifier_path, "rb") as f:
        clf = pickle.load(f)

    try:
        ann_tbl = ad.read_zarr(f"{zarr_url}/tables/{table_name}")
    except PathNotFoundError:
        raise FileNotFoundError(
            f"No measurements exist at the specified zarr URL in the table {table_name}."
        )
    features = ann_tbl.to_df()
    features_with_annotations = pd.concat(
        (ann_tbl.obs[["roi_id", "label"]], features), axis=1
    )
    predictions = clf.predict(features_with_annotations).reset_index()
    ann_tbl.obs = predictions
    ann_tbl.obs_names = ann_tbl.obs.index.map(str)
    ann_tbl.write_zarr(f"{zarr_url}/tables/{table_name}")
    io.write_table_metadata(zarr_url, "feature_table", table_name, label_name)


if __name__ == "__main__":
    from fractal_tasks_core.tasks._utils import run_fractal_task

    run_fractal_task(
        task_function=label_prediction,
        logger_name=logger.name,
    )
